// n505ve_audio 1999-12-14
// Andrew Green, ag@electricmagic.com
// http://www.electricmagic.com/~ag/snippets/n505ve_audio.html
// Derived from Joseph Wang's sbpro and Be Inc.'s sonic_vibes sample driver.

#ifndef __N505VE_Audio__
#define __N505VE_Audio__ 1

#include "ZDriver_Be.h"

#include <drivers/ISA.h> // For isa_module_info
#include <drivers/KernelExport.h> // For spinlock

#include "sound.h" // for sound_setup

// ==================================================

class N505VE_Audio_Driver : public ZDriver_Be_Driver
	{
public:
	static status_t sInitHardware();
	static status_t sInitDriver();

	N505VE_Audio_Driver();
	virtual ~N505VE_Audio_Driver();

	virtual bool Initialize();
	virtual void Finalize();
	};

// ==================================================

class N505VE_Audio_Card : public ZDriver_Be_Card
	{
public:
	N505VE_Audio_Card(int inPort, int inIRQ, int inDMA8);
	virtual ~N505VE_Audio_Card();

	virtual bool Initialize();
	virtual void Finalize();

private:
	int fPort;
	int fIRQ;
	int fDMA8;
	};

// ==================================================

class N505VE_Audio_DevicePCM : public ZDriver_Be_Device, public ZDriver_Be_DeviceInstance
	{
public:
	N505VE_Audio_DevicePCM(int inPort, int inIRQ, int inDMA8);
	~N505VE_Audio_DevicePCM();

// From ZDriver_Be_Device
	virtual bool Initialize();
	virtual void Finalize();

	virtual size_t CountNames();
	virtual const char* GetNameByIndex(size_t inNameIndex);

	virtual status_t Open(const char* inName, uint32 inFlags, ZDriver_Be_DeviceInstance*& outDeviceInstance);

// From ZDriver_Be_DeviceInstance
	virtual status_t Close();
	virtual status_t Free();
	virtual status_t Control(uint32 inOperation, void* inOutData, size_t inLen);
	virtual status_t Read(off_t inPos, void* outData, size_t& inOutLen);
	virtual status_t Write(off_t inPos, const void* inData, size_t& inOutLen);

// Our protocol
	status_t Internal_Write(const void* inData, size_t& inOutLen, bigtime_t& outTime, bigtime_t& outSampleTime);

	status_t SetParams(const sound_setup& inSoundSetup);
	status_t GetParams(sound_setup& outSoundSetup);

	void HandleInterrupt();

	void IncrementInterruptHandler();
	void DecrementInterruptHandler();

	static int32 sHandleInterrupt(void* inCookie);

	void StartDMA();
	void StopDMA();

	status_t WriteData(uint8 inValue);
	status_t ReadData(uint8& outValue);
	void WriteReg(uint8 inReg, uint8 inValue);
	uint8 ReadReg(uint8 inReg);
	void WriteIO(int inOffset, uint8 inValue);
	uint8 ReadIO(int inOffset);

	cpu_status DisableInterruptsAndAcquireSpinLock();
	void ReleaseSpinLockAndRestoreInterrupts(cpu_status inPriorStatus);
	
private:
	void* fWriteBuffer;
	int32 fWriteBufferSize;
	area_id fWriteBufferArea;

	unsigned char* fDoubleBuffer[2];
	size_t fDoubleBufferCurrent;
	size_t fDoubleBufferSize;

	sem_id fSem_Init;
	int32 fCount_Open;

	sem_id fSem_PlaybackComplete;
	sem_id fSem_Write;
	sem_id fSem_WriteEntry;
	
	int32 fCount_WriteWaiting;
	
	uint64 fSampleClock;
	bigtime_t fSampleTime;

	int32 fCountInterruptHandler;

	spinlock fSpinLock;

	isa_module_info* fISAModuleInfo;

	int fPort;
	int fIRQ;
	int fDMA8;
	bool fLockedDMA;
	};

// ==================================================

#endif // __N505VE_Audio___